MODULE closea
   !!======================================================================
   !!                   ***  MODULE  closea  ***
   !!
   !! User define : specific treatments associated with closed seas
   !!======================================================================
   !! History :   8.2  !  2000-05  (O. Marti)  Original code
   !!   NEMO      1.0  !  2002-06  (E. Durand, G. Madec)  F90
   !!             3.0  !  2006-07  (G. Madec)  add clo_rnf, clo_ups, clo_bat
   !!             3.4  !  2014-12  (P.G. Fogli) sbc_clo bug fix & mpp reproducibility
   !!             4.0  !  2016-06  (G. Madec)  move to usrdef_closea, remove clo_ups
   !!             4.0  !  2017-12  (D. Storkey) new formulation based on masks read from file
   !!             4.1  !  2019-07  (P. Mathiot) update to the new domcfg.nc input file
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   dom_clo    : read in masks which define closed seas and runoff areas
   !!   sbc_clo    : Special handling of freshwater fluxes over closed seas
   !!   clo_rnf    : set close sea outflows as river mouths (see sbcrnf)
   !!   clo_bat    : set to zero a field over closed sea (see domzgr)
   !!----------------------------------------------------------------------
   USE oce             ! dynamics and tracers
   USE dom_oce         ! ocean space and time domain
   USE phycst          ! physical constants
   USE sbc_oce         ! ocean surface boundary conditions
   USE iom             ! I/O routines
   !
   USE in_out_manager  ! I/O manager
   USE lib_fortran,    ONLY: glob_sum
   USE lbclnk          ! lateral boundary condition - MPP exchanges
   USE lib_mpp         ! MPP library
   USE timing          ! Timing

   IMPLICIT NONE
   PRIVATE

   PUBLIC dom_clo      ! called by domain module
   PUBLIC sbc_clo_init ! called by sbcmod module
   PUBLIC sbc_clo      ! called by sbcmod module
   PUBLIC clo_rnf      ! called by sbcrnf module
   PUBLIC clo_bat      ! called in domzgr module

   LOGICAL, PUBLIC :: ln_maskcs  !: mask all closed sea
   LOGICAL, PUBLIC :: ln_mask_csundef  !: mask all closed sea
   LOGICAL, PUBLIC :: ln_clo_rnf !: closed sea treated as runoff (update rnf mask)

   LOGICAL, PUBLIC :: l_sbc_clo  !: T => net evap/precip over closed seas spread outover the globe/river mouth
   LOGICAL, PUBLIC :: l_clo_rnf  !: T => Some closed seas output freshwater (RNF) to specified runoff points.

   INTEGER, PUBLIC :: jncsg      !: number of closed seas global mappings (inferred from closea_mask_glo field)
   INTEGER, PUBLIC :: jncsr      !: number of closed seas rnf    mappings (inferred from closea_mask_rnf field)
   INTEGER, PUBLIC :: jncse      !: number of closed seas empmr  mappings (inferred from closea_mask_emp field)

   INTEGER, PUBLIC, SAVE, ALLOCATABLE, DIMENSION(:)   :: jcsgrpg, jcsgrpr, jcsgrpe
  
   INTEGER, PUBLIC, SAVE, ALLOCATABLE, DIMENSION(:,:) :: mask_opnsea, mask_csundef 
 
   INTEGER, PUBLIC, SAVE, ALLOCATABLE, DIMENSION(:,:) ::  mask_csglo, mask_csgrpglo !: mask of integers defining closed seas
   INTEGER, PUBLIC, SAVE, ALLOCATABLE, DIMENSION(:,:) ::  mask_csrnf, mask_csgrprnf !: mask of integers defining closed seas rnf mappings
   INTEGER, PUBLIC, SAVE, ALLOCATABLE, DIMENSION(:,:) ::  mask_csemp, mask_csgrpemp !: mask of integers defining closed seas empmr mappings

   REAL(wp), PUBLIC, SAVE, ALLOCATABLE, DIMENSION(:) :: rsurfsrcg, rsurftrgg !: closed sea target glo surface areas 
   REAL(wp), PUBLIC, SAVE, ALLOCATABLE, DIMENSION(:) :: rsurfsrcr, rsurftrgr !: closed sea target rnf surface areas 
   REAL(wp), PUBLIC, SAVE, ALLOCATABLE, DIMENSION(:) :: rsurfsrce, rsurftrge !: closed sea target emp surface areas 

   !! * Substitutions
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OCE 4.0 , NEMO Consortium (2018)
   !! $Id$
   !! Software governed by the CeCILL license (see ./LICENSE)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE dom_clo()
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE dom_clo  ***
      !!        
      !! ** Purpose :   Closed sea domain initialization
      !!
      !! ** Method  :   if a closed sea is located only in a model grid point
      !!                just the thermodynamic processes are applied.
      !!
      !! ** Action  :   Read mask_cs* fields (if needed) from domain_cfg file and infer
      !!                number of closed seas for each case (glo, rnf, emp) from mask_cs* field.
      !!                mask_csglo and mask_csgrpglo  : integer values defining mappings from closed seas and associated groups to the open ocean for net fluxes.
      !!                mask_csrnf and mask_csgrprnf  : integer values defining mappings from closed seas and associated groups to a runoff area for downwards flux only.
      !!                mask_csemp and mask_csgrpemp  : integer values defining mappings from closed seas and associated groups to a runoff area for net fluxes.
      !!----------------------------------------------------------------------
      INTEGER ::   ios     ! io status
      !!
      NAMELIST/namclo/ ln_maskcs, ln_mask_csundef, ln_clo_rnf
      !!---------------------------------------------------------------------
      !!
      REWIND( numnam_ref )              ! Namelist namclo in reference namelist : Lateral momentum boundary condition
      READ  ( numnam_ref, namclo, IOSTAT = ios, ERR = 901 )
901   IF( ios /= 0 )   CALL ctl_nam ( ios , 'namclo in reference namelist', lwp )
      REWIND( numnam_cfg )              ! Namelist namclo in configuration namelist : Lateral momentum boundary condition
      READ  ( numnam_cfg, namclo, IOSTAT = ios, ERR = 902 )
902   IF( ios >  0 )   CALL ctl_nam ( ios , 'namclo in configuration namelist', lwp )
      IF(lwm) WRITE ( numond, namclo )
      !!
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*)'dom_clo : read in masks to define closed seas '
      IF(lwp) WRITE(numout,*)'~~~~~~~'
      !!
      !! check option compatibility
      IF( .NOT. ln_read_cfg ) THEN
         CALL ctl_stop('Suppression of closed seas does not work with ln_read_cfg = .true. . Set ln_closea = .false. .')
      ENDIF
      !!
      IF( (.NOT. ln_maskcs) .AND. ln_diurnal_only ) THEN
         CALL ctl_stop('Special handling of freshwater fluxes over closed seas not compatible with ln_diurnal_only.')
      END IF
      !
      l_sbc_clo = .false. ; l_clo_rnf = .false.
      IF (.NOT. ln_maskcs)                  l_sbc_clo = .true.
      IF (.NOT. ln_maskcs .AND. ln_clo_rnf) l_clo_rnf = .true.
      !
      ! read the closed seas masks (if they exist) from domain_cfg file (if it exists)
      ! ------------------------------------------------------------------------------
      !
      ! load mask of open sea and undefined closed seas
      CALL alloc_csmask( mask_opnsea )
      CALL read_csmask( cn_domcfg, 'mask_opensea' , mask_opnsea  )
      !
      IF ( ln_maskcs ) THEN
         ! not special treatment of closed sea
         l_sbc_clo = .false. ; l_clo_rnf = .false.
      ELSE
         ! special treatment of closed seas
         l_sbc_clo = .true.
         !
         ! river mouth from lakes added to rnf mask for special treatment
         IF ( ln_clo_rnf) l_clo_rnf = .true.
         !
         IF ( ln_mask_csundef) THEN
            ! load undef cs mask (1 in undef closed sea)
            CALL alloc_csmask( mask_csundef )
            CALL read_csmask( cn_domcfg, 'mask_csundef', mask_csundef )
            ! revert the mask for masking in domzgr
            mask_csundef = 1 - mask_csundef
         END IF
         !
         ! allocate source mask
         CALL alloc_csmask( mask_csglo )
         CALL alloc_csmask( mask_csrnf )
         CALL alloc_csmask( mask_csemp )
         !
         ! load source mask of cs for each cases
         CALL read_csmask( cn_domcfg, 'mask_csglo', mask_csglo )
         CALL read_csmask( cn_domcfg, 'mask_csrnf', mask_csrnf )
         CALL read_csmask( cn_domcfg, 'mask_csemp', mask_csemp )
         !
         ! compute number of cs for each cases
         jncsg = MAXVAL( mask_csglo(:,:) ) ; CALL mpp_max( 'closea', jncsg )
         jncsr = MAXVAL( mask_csrnf(:,:) ) ; CALL mpp_max( 'closea', jncsr )
         jncse = MAXVAL( mask_csemp(:,:) ) ; CALL mpp_max( 'closea', jncse )
         !
         ! allocate closed sea group masks
         CALL alloc_csmask( mask_csgrpglo )
         CALL alloc_csmask( mask_csgrprnf )
         CALL alloc_csmask( mask_csgrpemp )

         ! load mask of cs group for each cases
         CALL read_csmask( cn_domcfg, 'mask_csgrpglo', mask_csgrpglo )
         CALL read_csmask( cn_domcfg, 'mask_csgrprnf', mask_csgrprnf )
         CALL read_csmask( cn_domcfg, 'mask_csgrpemp', mask_csgrpemp )
         !
         ! Allocate cs variables (surf)
         CALL alloc_cssurf( jncsg, rsurfsrcg, rsurftrgg ) 
         CALL alloc_cssurf( jncsr, rsurfsrcr, rsurftrgr )
         CALL alloc_cssurf( jncse, rsurfsrce, rsurftrge )
         !
         ! Allocate cs group variables (jcsgrp)
         CALL alloc_csgrp( jncsg, jcsgrpg )
         CALL alloc_csgrp( jncsr, jcsgrpr )
         CALL alloc_csgrp( jncse, jcsgrpe )
         !
      END IF
   END SUBROUTINE dom_clo

   SUBROUTINE sbc_clo_init

      ! compute source surface area
      CALL get_cssrcsurf( jncsg, mask_csglo, rsurfsrcg )
      CALL get_cssrcsurf( jncsr, mask_csrnf, rsurfsrcr )
      CALL get_cssrcsurf( jncse, mask_csemp, rsurfsrce )
      !
      ! compute target surface area and group number (jcsgrp) for all cs and cases 
      ! glo could be simpler but for lisibility, all treated the same way
      ! It is only done once, so not a big deal
      CALL get_cstrgsurf( jncsg, mask_csglo, mask_csgrpglo, rsurftrgg, jcsgrpg )
      CALL get_cstrgsurf( jncsr, mask_csrnf, mask_csgrprnf, rsurftrgr, jcsgrpr )
      CALL get_cstrgsurf( jncse, mask_csemp, mask_csgrpemp, rsurftrge, jcsgrpe )
      ! 
      ! print out in ocean.ouput
      CALL prt_csctl( jncsg, rsurfsrcg, rsurftrgg, jcsgrpg, 'glo' )
      CALL prt_csctl( jncsr, rsurfsrcr, rsurftrgr, jcsgrpr, 'rnf' )
      CALL prt_csctl( jncse, rsurfsrce, rsurftrge, jcsgrpe, 'emp' )

   END SUBROUTINE sbc_clo_init

   SUBROUTINE get_cssrcsurf(kncs, pmaskcs, psurfsrc)

      ! subroutine parameters
      INTEGER,                 INTENT(in   ) :: kncs
      INTEGER, DIMENSION(:,:), INTENT(in   ) :: pmaskcs

      REAL(wp), DIMENSION(:)  , INTENT(inout) :: psurfsrc

      ! local variables
      INTEGER, DIMENSION(jpi,jpj) :: zmsksrc
      INTEGER :: jcs

      DO jcs = 1,kncs
         !
         ! build river mouth mask for this lake
         WHERE ( pmaskcs == jcs )
            zmsksrc = 1
         ELSE WHERE
            zmsksrc = 0
         END WHERE
         !
         ! compute target area
         psurfsrc(jcs) = glob_sum('closea', e1e2t(:,:) * zmsksrc(:,:) )
         !
      END DO

   END SUBROUTINE

   SUBROUTINE get_cstrgsurf(kncs, pmaskcs, pmaskcsgrp, psurftrg, kcsgrp )

      ! subroutine parameters
      INTEGER,               INTENT(in   ) :: kncs
      INTEGER, DIMENSION(:), INTENT(inout) :: kcsgrp
      INTEGER, DIMENSION(:,:), INTENT(in   ) :: pmaskcs, pmaskcsgrp

      REAL(wp), DIMENSION(:)  , INTENT(inout) :: psurftrg

      ! local variables
      INTEGER, DIMENSION(jpi,jpj) :: zmskgrp, zmsksrc, zmsktrg
      INTEGER :: jcs, jtmp

      DO jcs = 1,kncs
         !
         ! find group number
         zmskgrp = pmaskcsgrp
         zmsksrc = pmaskcs
         !
         ! set value where cs is
         zmsktrg = HUGE(1)
         WHERE ( zmsksrc == jcs ) zmsktrg = jcs
         !
         ! zmsk = HUGE outside the cs number jcs
         ! ktmp = jcs - group number
         ! jgrp = group corresponding to the cs jcs
         zmsktrg = zmsktrg - zmskgrp
         jtmp = MINVAL(zmsktrg) ; CALL mpp_min('closea',jtmp)
         kcsgrp(jcs) = jcs - jtmp
         !
         ! build river mouth mask for this lake
         WHERE ( zmskgrp * mask_opnsea == kcsgrp(jcs) )
            zmsktrg = 1
         ELSE WHERE
            zmsktrg = 0
         END WHERE
         !
         ! compute target area
         psurftrg(jcs) = glob_sum('closea', e1e2t(:,:) * zmsktrg(:,:) )
         !
      END DO

   END SUBROUTINE

   SUBROUTINE prt_csctl(kncs, psurfsrc, psurftrg, kcsgrp, pcstype)
      ! subroutine parameters
      INTEGER, INTENT(in   ) :: kncs
      INTEGER, DIMENSION(:), INTENT(in   ) :: kcsgrp
      !
      REAL(wp), DIMENSION(:), INTENT(in   ) :: psurfsrc, psurftrg
      !
      CHARACTER(256), INTENT(in   ) :: pcstype
      !
      ! local variable
      INTEGER :: jcs
      
      IF ( lwp .AND. kncs > 0 ) THEN
         WRITE(numout,*)''
         !
         WRITE(numout,*)'Closed sea target ',TRIM(pcstype),' : '
         !
         DO jcs = 1,kncs
            WRITE(numout,FMT='(3a,i3,a,i3)') ' ',TRIM(pcstype),' closed sea id is ',jcs,' and trg id is : ', kcsgrp(jcs)
            WRITE(numout,FMT='(a,f12.2)'   ) ' src surface areas (km2) : ', psurfsrc(jcs) * 1.0e-6
            WRITE(numout,FMT='(a,f12.2)'   ) ' trg surface areas (km2) : ', psurftrg(jcs) * 1.0e-6
         END DO
         !
         WRITE(numout,*)''
      END IF
   END SUBROUTINE

   SUBROUTINE sbc_clo( kt ) ! to be move in SBC in a file sbcclo ???
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE sbc_clo  ***
      !!                    
      !! ** Purpose :   Special handling of closed seas
      !!
      !! ** Method  :   Water flux is forced to zero over closed sea
      !!      Excess is shared between remaining ocean, or
      !!      put as run-off in open ocean.
      !!
      !! ** Action  :   emp updated surface freshwater fluxes and associated heat content at kt
      !!----------------------------------------------------------------------
      INTEGER         , INTENT(in   ) ::   kt       ! ocean model time step
      !!----------------------------------------------------------------------
      !
      IF( ln_timing )  CALL timing_start('sbc_clo')
      !
      ! update emp and qns
      CALL sbc_csupdate( jncsg, jcsgrpg, mask_csglo, mask_csgrpglo, rsurfsrcg, rsurftrgg, 'glo', mask_opnsea, rsurftrgg )
      CALL sbc_csupdate( jncsr, jcsgrpr, mask_csrnf, mask_csgrprnf, rsurfsrcr, rsurftrgr, 'rnf', mask_opnsea, rsurftrgg )
      CALL sbc_csupdate( jncse, jcsgrpe, mask_csemp, mask_csgrpemp, rsurfsrce, rsurftrge, 'emp', mask_opnsea, rsurftrgg )
      !
      ! is this really useful ??????
      emp(:,:) = emp(:,:) * tmask(:,:,1)
      qns(:,:) = qns(:,:) * tmask(:,:,1)
      !
      ! is this really useful ??????
      CALL lbc_lnk( 'closea', emp , 'T', 1._wp )
      CALL lbc_lnk( 'closea', qns , 'T', 1._wp )
      !
   END SUBROUTINE sbc_clo
   
   SUBROUTINE sbc_csupdate(kncs, kcsgrp, pmsk_src, pmsk_trg, psurfsrc, psurftrg, pcstype, pmsk_opnsea, psurf_opnsea)

      ! subroutine parameters
      INTEGER, INTENT(in   ) :: kncs
      INTEGER, DIMENSION(:  ), INTENT(in   ) :: kcsgrp
      INTEGER, DIMENSION(:,:), INTENT(in   ) :: pmsk_src, pmsk_trg, pmsk_opnsea
      
      REAL(wp), DIMENSION(:), INTENT(inout) :: psurfsrc, psurftrg, psurf_opnsea

      CHARACTER(256), INTENT(in   ) :: pcstype

      ! local variables
      INTEGER :: jcs
      INTEGER, DIMENSION(jpi,jpj) :: zmsk_src, zmsk_trg
      
      REAL(wp) :: zcoef, zcoef1, ztmp
      REAL(wp) :: zcsfwf
      REAL(wp) :: zsurftrg
     
      DO jcs = 1, kncs
         !!
         !! 0. get mask of each closed sea
         zmsk_src(:,:) = 0
         WHERE ( pmsk_src(:,:) == jcs ) zmsk_src = 1
         !!
         !! 1. Work out net freshwater fluxes over each closed seas from EMP - RNF.
         zcsfwf = glob_sum( 'closea', e1e2t(:,:) * ( emp(:,:)-rnf(:,:) ) * zmsk_src )
         !!
         !! 2. Deal with runoff special case (net evaporation spread globally)
         IF (pcstype == 'rnf' .AND. zcsfwf > 0) THEN
            zsurftrg = psurf_opnsea(1)
            zmsk_trg = pmsk_opnsea
         ELSE
            zsurftrg = psurftrg(jcs)
            zmsk_trg = pmsk_trg
         END IF
         zmsk_trg = zmsk_trg * pmsk_opnsea
         !!
         !! 3. Add residuals to target points
         zcoef    = zcsfwf / zsurftrg
         zcoef1   = rcp * zcoef
         WHERE( zmsk_trg(:,:) == kcsgrp(jcs) )
            emp(:,:) = emp(:,:) + zcoef
            qns(:,:) = qns(:,:) - zcoef1 * sst_m(:,:)
         ENDWHERE
         !!
         !! 4. Subtract residuals from source points
         zcoef    = zcsfwf / psurfsrc(jcs)
         zcoef1   = rcp * zcoef
         WHERE( pmsk_src(:,:) == jcs )
            emp(:,:) = emp(:,:) - zcoef
            qns(:,:) = qns(:,:) + zcoef1 * sst_m(:,:)
         ENDWHERE
         !!
      END DO ! jcs

   END SUBROUTINE


   SUBROUTINE clo_rnf( p_rnfmsk )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE clo_rnf  ***
      !!                    
      !! ** Purpose :   allow the treatment of closed sea outflow grid-points
      !!                to be the same as river mouth grid-points
      !!
      !! ** Method  :   set to 1 the runoff mask (mskrnf, see sbcrnf module)
      !!                at the closed sea outflow grid-point.
      !!
      !! ** Action  :   update (p_)mskrnf (set 1 at closed sea outflow)
      !!----------------------------------------------------------------------
      !!
      !! subroutine parameter
      REAL(wp), DIMENSION(jpi,jpj), INTENT(inout) ::   p_rnfmsk   ! river runoff mask (rnfmsk array)
      !!
      !! local variables
      REAL(wp), DIMENSION(jpi,jpj) :: zmsk
      !!----------------------------------------------------------------------
      !
      ! zmsk > 0 where cs river mouth defined (case rnf and emp)
      zmsk(:,:) = ( mask_csgrprnf (:,:) + mask_csgrpemp(:,:) ) * mask_opnsea(:,:)
      WHERE( zmsk(:,:) > 0 )
         p_rnfmsk(:,:) = 1.0_wp
      END WHERE
      !
   END SUBROUTINE clo_rnf
      
   SUBROUTINE clo_bat( k_top, k_bot, p_mask, p_prt )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE clo_bat  ***
      !!                    
      !! ** Purpose :   Suppress closed sea from the domain
      !!
      !! ** Method  :   Read in closea_mask field (if it exists) from domain_cfg file.
      !!                Where closea_mask > 0 set first and last ocean level to 0
      !!                (As currently coded you can't define a closea_mask field in 
      !!                usr_def_zgr).
      !!
      !! ** Action  :   set k_top=0 and k_bot=0 over closed seas
      !!----------------------------------------------------------------------
      !!
      !! subroutine parameter
      INTEGER, DIMENSION(:,:), INTENT(inout) ::   k_top, k_bot   ! ocean first and last level indices
      INTEGER, DIMENSION(:,:), INTENT(in   ) ::   p_mask         ! mask used to mask ktop and k_bot
      CHARACTER(256),          INTENT(in   ) ::   p_prt          ! text for control print
      !!
      !! local variables
      !!----------------------------------------------------------------------
      !!
      IF ( lwp ) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'clo_bat : Suppression closed seas based on ',TRIM(p_prt),' field.'
         WRITE(numout,*) '~~~~~~~'
         WRITE(numout,*)
      ENDIF
      !!
      k_top(:,:) = k_top(:,:) * p_mask(:,:)
      k_bot(:,:) = k_bot(:,:) * p_mask(:,:)
      !!
   END SUBROUTINE clo_bat

   SUBROUTINE read_csmask(p_file, p_var, p_mskout)
      !
      ! subroutine parameter
      CHARACTER(256), INTENT(in   ) :: p_file, p_var
      INTEGER, DIMENSION(:,:), INTENT(inout) :: p_mskout
      !
      ! local variables
      INTEGER :: ics
      REAL(wp), DIMENSION(jpi,jpj) :: zdta
      !
      CALL iom_open ( p_file, ics )
      CALL iom_get  ( ics, jpdom_data, TRIM(p_var), zdta )
      CALL iom_close( ics )
      p_mskout(:,:) = NINT(zdta(:,:))
      !
   END SUBROUTINE read_csmask

   SUBROUTINE alloc_csmask( pmask )
      !
      ! subroutine parameter
      INTEGER, ALLOCATABLE, DIMENSION(:,:), INTENT(inout) :: pmask
      !
      ! local variables
      INTEGER :: ierr
      !
      ALLOCATE( pmask(jpi,jpj) , STAT=ierr )
      IF( ierr /= 0 )   CALL ctl_stop( 'STOP', 'alloc_csmask: failed to allocate surf array')
      !
   END SUBROUTINE


   SUBROUTINE alloc_cssurf( klen, pvarsrc, pvartrg )
      !
      ! subroutine parameter
      INTEGER, INTENT(in) :: klen
      REAL(wp), ALLOCATABLE, DIMENSION(:), INTENT(inout) :: pvarsrc, pvartrg 
      !
      ! local variables
      INTEGER :: ierr
      !
      ! klen (number of lake) can be zero so use MAX(klen,1) to avoid 0 length array
      ALLOCATE( pvarsrc(MAX(klen,1)) , pvartrg(MAX(klen,1)) , STAT=ierr )
      IF( ierr /= 0 )   CALL ctl_stop( 'STOP', 'sbc_clo: failed to allocate surf array')
      ! initialise to 0
      pvarsrc(:) = 0.e0_wp
      pvartrg(:) = 0.e0_wp
   END SUBROUTINE

   SUBROUTINE alloc_csgrp( klen, kvar )
      !
      ! subroutine parameter
      INTEGER, INTENT(in) :: klen
      INTEGER, ALLOCATABLE, DIMENSION(:), INTENT(inout) :: kvar 
      !
      ! local variables
      INTEGER :: ierr
      !
      ! klen (number of lake) can be zero so use MAX(klen,1) to avoid 0 length array
      ALLOCATE( kvar(MAX(klen,1)) , STAT=ierr )
      IF( ierr /= 0 )   CALL ctl_stop( 'STOP', 'sbc_clo: failed to allocate group array')
      ! initialise to 0
      kvar(:) = 0
   END SUBROUTINE

   !!======================================================================
END MODULE closea

